@@ -182,7 +182,6 @@ pub use paste::paste as paste_paste;
 use crate::binding::*;
 use alloc::boxed::Box;
 use alloc::collections::TryReserveError;
-use alloc::vec;
 use alloc::vec::Vec;
 use components_arena::{Arena, Component, ComponentId, Id};
 use core::any::{Any, TypeId};
@@ -190,6 +189,7 @@ use core::fmt::Debug;
 use core::mem::{replace, take};
 use core::ops::{Deref, DerefMut};
 use dyn_clone::{DynClone, clone_trait_object};
+use dyn_context::free_lifetimes;
 use dyn_context::state::State;
 use educe::Educe;
 use macro_attr_2018::macro_attr;
@@ -318,11 +318,26 @@ impl<ArgsType> DepEventEntry<ArgsType> {
     }
 }
 
+free_lifetimes! {
+    #[derive(Debug)]
+    pub struct Items<ItemType> {
+        items: 'items ref [ItemType],
+    }
+}
+
+impl<ItemType> Deref for Items<ItemType> {
+    type Target = [ItemType];
+
+    fn deref(&self) -> &Self::Target {
+        self.items()
+    }
+}
+
 #[derive(Debug)]
 pub struct DepVecEntry<ItemType: Convenient> {
     items: Vec<ItemType>,
-    removed_items_handlers: Arena<BoxedValueHandler<Vec<ItemType>>>,
-    inserted_items_handlers: Arena<BoxedValueHandler<Vec<ItemType>>>,
+    removed_items_handlers: Arena<BoxedEventHandler<Items<ItemType>>>,
+    inserted_items_handlers: Arena<BoxedEventHandler<Items<ItemType>>>,
 }
 
 impl<ItemType: Convenient> DepVecEntry<ItemType> {
@@ -836,9 +851,13 @@ impl<Owner: DepType, ItemType: Convenient> DepVec<Owner, ItemType> {
         let entry_mut = self.entry_mut(&mut obj_mut);
         let items = take(&mut entry_mut.items);
         let handlers = entry_mut.removed_items_handlers.items().clone().into_values();
-        for handler in handlers {
-            handler.0.execute(state, items.clone());
-        }
+        ItemsBuilder {
+            items: &items
+        }.build_and_then(|items| {
+            for handler in handlers {
+                handler.0.execute(state, items);
+            }
+        });
     }
 
     pub fn push(self, state: &mut dyn State, obj: Glob<Owner::Id, Owner>, item: ItemType) {
@@ -846,9 +865,13 @@ impl<Owner: DepType, ItemType: Convenient> DepVec<Owner, ItemType> {
         let entry_mut = self.entry_mut(&mut obj_mut);
         entry_mut.items.push(item.clone());
         let handlers = entry_mut.inserted_items_handlers.items().clone().into_values();
-        for handler in handlers {
-            handler.0.execute(state, vec![item.clone()]);
-        }
+        ItemsBuilder {
+            items: &[item]
+        }.build_and_then(|items| {
+            for handler in handlers {
+                handler.0.execute(state, items);
+            }
+        });
     }
 
     pub fn insert(self, state: &mut dyn State, obj: Glob<Owner::Id, Owner>, index: usize, item: ItemType) {
@@ -856,9 +879,13 @@ impl<Owner: DepType, ItemType: Convenient> DepVec<Owner, ItemType> {
         let entry_mut = self.entry_mut(&mut obj_mut);
         entry_mut.items.insert(index, item.clone());
         let handlers = entry_mut.inserted_items_handlers.items().clone().into_values();
-        for handler in handlers {
-            handler.0.execute(state, vec![item.clone()]);
-        }
+        ItemsBuilder {
+            items: &[item]
+        }.build_and_then(|items| {
+            for handler in handlers {
+                handler.0.execute(state, items);
+            }
+        });
     }
 
     pub fn remove(self, state: &mut dyn State, obj: Glob<Owner::Id, Owner>, index: usize) {
@@ -866,9 +893,13 @@ impl<Owner: DepType, ItemType: Convenient> DepVec<Owner, ItemType> {
         let entry_mut = self.entry_mut(&mut obj_mut);
         let item = entry_mut.items.remove(index);
         let handlers = entry_mut.removed_items_handlers.items().clone().into_values();
-        for handler in handlers {
-            handler.0.execute(state, vec![item.clone()]);
-        }
+        ItemsBuilder {
+            items: &[item]
+        }.build_and_then(|items| {
+            for handler in handlers {
+                handler.0.execute(state, items);
+            }
+        });
     }
 
     pub fn extend_from_slice(self, state: &mut dyn State, obj: Glob<Owner::Id, Owner>, other: &[ItemType]) {
@@ -876,9 +907,13 @@ impl<Owner: DepType, ItemType: Convenient> DepVec<Owner, ItemType> {
         let entry_mut = self.entry_mut(&mut obj_mut);
         entry_mut.items.extend_from_slice(other);
         let handlers = entry_mut.inserted_items_handlers.items().clone().into_values();
-        for handler in handlers {
-            handler.0.execute(state, Vec::from(other));
-        }
+        ItemsBuilder {
+            items: other
+        }.build_and_then(|items| {
+            for handler in handlers {
+                handler.0.execute(state, items);
+            }
+        });
     }
 
     pub fn inserted_items_source(self, obj: Glob<Owner::Id, Owner>) -> DepVecInsertedItemSource<Owner, ItemType> {
@@ -1087,14 +1122,20 @@ pub struct DepEventSource<Owner: DepType, ArgsType> {
 }
 
 impl<Owner: DepType + 'static, ArgsType: 'static> EventSource<ArgsType> for DepEventSource<Owner, ArgsType> {
-    fn handle(&self, state: &mut dyn State, handler: Box<dyn EventHandler<ArgsType>>) -> HandledEventSource<ArgsType> {
+    fn handle(
+        &self,
+        state: &mut dyn State,
+        handler: Box<dyn EventHandler<ArgsType>>,
+        result: Box<dyn FnOnce(HandledEventSource<ArgsType>)>
+    ) {
         let mut obj = self.obj.get_mut(state);
         let entry = self.event.entry_mut(&mut obj);
         let handler_id = entry.handlers.insert(|handler_id| (BoxedEventHandler(handler), handler_id));
-        HandledEventSource {
+        result(HandledEventSource {
+            state,
             handler_id: Box::new(DepEventHandledSource { handler_id, obj: self.obj, event: self.event }),
             args: None // TODO "once" events with cached value?
-        }
+        })
     }
 }
 
@@ -1139,7 +1180,7 @@ impl<Owner: DepType + 'static, PropType: Convenient> ValueSource<(PropType, Prop
 #[educe(Debug)]
 struct DepVecInsertedItemsHandledSource<Owner: DepType, ItemType: Convenient> {
     obj: Glob<Owner::Id, Owner>,
-    handler_id: Id<BoxedValueHandler<Vec<ItemType>>>,
+    handler_id: Id<BoxedEventHandler<Items<ItemType>>>,
     vec: DepVec<Owner, ItemType>,
 }
 
@@ -1155,7 +1196,7 @@ impl<Owner: DepType, ItemType: Convenient> HandlerId for DepVecInsertedItemsHand
 #[educe(Debug)]
 struct DepVecRemovedItemsHandledSource<Owner: DepType, ItemType: Convenient> {
     obj: Glob<Owner::Id, Owner>,
-    handler_id: Id<BoxedValueHandler<Vec<ItemType>>>,
+    handler_id: Id<BoxedEventHandler<Items<ItemType>>>,
     vec: DepVec<Owner, ItemType>,
 }
 
@@ -1174,16 +1215,26 @@ pub struct DepVecInsertedItemSource<Owner: DepType, ItemType: Convenient> {
     vec: DepVec<Owner, ItemType>,
 }
 
-impl<Owner: DepType + 'static, ItemType: Convenient> ValueSource<Vec<ItemType>> for DepVecInsertedItemSource<Owner, ItemType> {
-    fn handle(&self, state: &mut dyn State, handler: Box<dyn ValueHandler<Vec<ItemType>>>) -> HandledValueSource<Vec<ItemType>> {
+impl<Owner: DepType + 'static, ItemType: Convenient> EventSource<Items<ItemType>> for DepVecInsertedItemSource<Owner, ItemType> {
+    fn handle(
+        &self,
+        state: &mut dyn State,
+        handler: Box<dyn EventHandler<Items<ItemType>>>,
+        result: Box<dyn FnOnce(HandledEventSource<Items<ItemType>>)>
+    ) {
         let mut obj = self.obj.get_mut(state);
         let entry = self.vec.entry_mut(&mut obj);
         let items = entry.items.clone();
-        let handler_id = entry.inserted_items_handlers.insert(|handler_id| (BoxedValueHandler(handler), handler_id));
-        HandledValueSource {
-            handler_id: Box::new(DepVecInsertedItemsHandledSource { handler_id, obj: self.obj, vec: self.vec }),
-            value: items
-        }
+        let handler_id = entry.inserted_items_handlers.insert(|handler_id| (BoxedEventHandler(handler), handler_id));
+        ItemsBuilder {
+            items: &items
+        }.build_and_then(|items| {
+            result(HandledEventSource {
+                state,
+                handler_id: Box::new(DepVecInsertedItemsHandledSource { handler_id, obj: self.obj, vec: self.vec }),
+                args: Some(items)
+            })
+        });
     }
 }
 
@@ -1194,16 +1245,21 @@ pub struct DepVecRemovedItemSource<Owner: DepType, ItemType: Convenient> {
     vec: DepVec<Owner, ItemType>,
 }
 
-impl<Owner: DepType + 'static, ItemType: Convenient> ValueSource<Vec<ItemType>> for DepVecRemovedItemSource<Owner, ItemType> {
-    fn handle(&self, state: &mut dyn State, handler: Box<dyn ValueHandler<Vec<ItemType>>>) -> HandledValueSource<Vec<ItemType>> {
+impl<Owner: DepType + 'static, ItemType: Convenient> EventSource<Items<ItemType>> for DepVecRemovedItemSource<Owner, ItemType> {
+    fn handle(
+        &self,
+        state: &mut dyn State,
+        handler: Box<dyn EventHandler<Items<ItemType>>>,
+        result: Box<dyn FnOnce(HandledEventSource<Items<ItemType>>)>
+    ) {
         let mut obj = self.obj.get_mut(state);
         let entry = self.vec.entry_mut(&mut obj);
-        let items = entry.items.clone();
-        let handler_id = entry.removed_items_handlers.insert(|handler_id| (BoxedValueHandler(handler), handler_id));
-        HandledValueSource {
+        let handler_id = entry.removed_items_handlers.insert(|handler_id| (BoxedEventHandler(handler), handler_id));
+        result(HandledEventSource {
+            state,
             handler_id: Box::new(DepVecRemovedItemsHandledSource { handler_id, obj: self.obj, vec: self.vec }),
-            value: items
-        }
+            args: None
+        })
     }
 }
 
